#include "WatchFace.h"
#include "SevenSegment.h"
#include "Icons.h"
#include <cmath>
#include <Fonts/FreeSans9pt7b.h>
#include <Fonts/FreeMonoBold9pt7b.h>
#include <Fonts/FreeMonoBold12pt7b.h>

RTC_DATA_ATTR bool WatchFace::m_menuSetup = false;
RTC_DATA_ATTR bool WatchFace::m_inMenu = false;
RTC_DATA_ATTR int WatchFace::m_tzOffset = TZ_OFFSET;
Menu WatchFace::m_menu;

void WatchFace::Setup() // Called after hardware is set up
{
	if (!m_menuSetup) {
		m_menu.Init(m_display);
	}

	SetupVolatileMenuStuff();

	if (!m_menuSetup) {
		m_menu.Reset();
		m_menuSetup = true;
	}
}

void WatchFace::HandleButtonPress(uint64_t buttonMask)
{
  if (m_inMenu) {
    m_menu.HandleButtonPress(buttonMask);
  } else {
    if (buttonMask & MENU_BTN_MASK) {
      m_inMenu = true;
      m_menu.Redraw(false);

      const std::vector<int> buttons = {MENU_BTN_PIN, UP_BTN_PIN, DOWN_BTN_PIN, BACK_BTN_PIN};
      
      while (m_inMenu) {
        for (int button : buttons) {
          if (digitalRead(button) == HIGH) {
            switch (button) {
              case MENU_BTN_PIN:
                m_menu.HandleButtonPress(MENU_BTN_MASK);
                break;
              case UP_BTN_PIN:
                m_menu.HandleButtonPress(UP_BTN_MASK);
                break;
              case DOWN_BTN_PIN:
                m_menu.HandleButtonPress(DOWN_BTN_MASK);
                break;
              case BACK_BTN_PIN:
                m_menu.HandleButtonPress(BACK_BTN_MASK);
                break;
            }

            while (true) {
              if (digitalRead(button) == LOW) {
                break;
              }

              delay(10);
            }
          }
        }

        delay(10);
      }
    }
  }
}

void WatchFace::HandleDoubleTap()
{
  ShowWatchFace(true);
}

void WatchFace::HandleTilt()
{
  ShowWatchFace(true);
}

void WatchFace::DrawWatchFace(bool partialRefresh)
{
  if (m_inMenu) {
    return;
  }

  m_display.setFullWindow();

  m_display.fillScreen(GxEPD_WHITE);
  DrawBatteryIcon();

	tmElements_t currentTime;
	m_RTC.read(currentTime, m_tzOffset);
	SevenSegment sevenSegment(30, 60, 6, 5, 5);

  if (currentTime.Hour < 10) {
    sevenSegment.DrawDigit(m_display, 0, 15, 75, GxEPD_BLACK);
  } else {
    sevenSegment.DrawDigit(m_display, currentTime.Hour / 10, 20, 75, GxEPD_BLACK);
  }

  sevenSegment.DrawDigit(m_display, currentTime.Hour % 10, 60, 75, GxEPD_BLACK);

  if (currentTime.Minute < 10) {
    sevenSegment.DrawDigit(m_display, 0, 110, 75, GxEPD_BLACK);
  } else {
    sevenSegment.DrawDigit(m_display, currentTime.Minute / 10, 110, 75, GxEPD_BLACK);
  }

  sevenSegment.DrawDigit(m_display, currentTime.Minute % 10, 150, 75, GxEPD_BLACK);

  m_display.fillRect(97, 90, 5, 5, GxEPD_BLACK);
  m_display.fillRect(97, 110, 5, 5, GxEPD_BLACK);

  m_display.display(partialRefresh);
}

void WatchFace::DrawBatteryIcon()
{
  m_display.fillRect(200 - 48, 5, 43, 23, GxEPD_BLACK);
  m_display.fillRect(200 - 46, 7, 39, 19, GxEPD_WHITE);
  float VBAT = GetBatteryVoltage();
  float level = (VBAT - 3.6f) / 0.6f;
  if (level > 1.0f) {
    level = 1.0f;
  } else if (level < 0.0f) {
    level = 0.0f;
  }
  level = 0.5f - sin(asin(1.0f - 2.0f * level) / 3.0f);

  m_display.fillRect(200 - 44, 9, (int)std::round(35.0f * level), 15, GxEPD_BLACK);

  int x = 200 - 85;
  int intLevel = (int)std::round(100.0f * level);
  if (intLevel == 100) {
    x -= 13;
  }

  m_display.setFont(&FreeMonoBold9pt7b);
  m_display.setCursor(x, 22);
  m_display.setTextColor(GxEPD_BLACK);
  m_display.print(intLevel);
  m_display.print("%");

  m_display.setFont(&FreeMonoBold12pt7b);
  m_display.drawBitmap(10, 177, Icons::steps, 19, 23, GxEPD_BLACK);
  m_display.setCursor(40, 195);
  m_display.print(GetSteps());
}

void WatchFace::SetupVolatileMenuStuff()
{
	static const std::vector<MenuPage> menuPages = {
		{
			0, // backPageNum
			"WATCHY", // title
			ALIGNMENT_CENTER, // titleAlignment
			"", // body
			{ // Menu items
				{
					"Sync NTP", // title
					nullptr, // callback
					1 // pageNum
				},
				{
					"Set Timezone", // title
					nullptr, // callback
					2 // pageNum
				},
        {
          "Reset Steps", // title
          nullptr, // callback
          3 // pageNum
        },
				{
					"Back", // title
					nullptr, // callback
					0 // pageNum
				}
			},
		},
		{
			0, // backPageNum
			"SYNC NTP", // title
			ALIGNMENT_CENTER, // titleAlignment
			"Sync with:\n" NTP_SERVER, // body
			{ // Menu items
				{
					"Sync", // title
					std::bind(&WatchFace::MenuNTPSyncSelected, this), // callback
					1 // pageNum
				},
				{
					"Back", // title
					nullptr, // callback
					0 // pageNum
				}
			},
		},
		{
			0, // backPageNum
			"TIMEZONE", // title
			ALIGNMENT_CENTER, // titleAlignment
			"", // body
			{ // Menu items
				{
					"+0", // title
					std::bind(&WatchFace::MenuTimeZoneSelected, this, 0), // callback
					0 // pageNum
				},
				{
					"+1", // title
					std::bind(&WatchFace::MenuTimeZoneSelected, this, 3600), // callback
					0 // pageNum
				},
				{
					"+2", // title
					std::bind(&WatchFace::MenuTimeZoneSelected, this, 7200), // callback
					0 // pageNum
				},
				{
					"+3", // title
					std::bind(&WatchFace::MenuTimeZoneSelected, this, 10800), // callback
					0 // pageNum
				},
				{
					"Back", // title
					nullptr, // callback
					0 // pageNum
				}
			},
		},
		{
			0, // backPageNum
			"RESET STEPS", // title
			ALIGNMENT_CENTER, // titleAlignment
			"Confirm?", // body
			{ // Menu items
				{
					"No", // title
					nullptr, // callback
					0 // pageNum
				},
				{
					"Yes", // title
					std::bind(&WatchFace::MenuConfirmResetSteps, this), // callback
					0 // pageNum
				}
			}
		}
	};

  m_menu.SetPages(menuPages);
  m_menu.SetExitCallback(std::bind(&WatchFace::MenuExited, this));
}

void WatchFace::MenuExited()
{
  if (m_inMenu) {
    m_inMenu = false;
    DrawWatchFace(false);
  }
}

void WatchFace::MenuNTPSyncSelected()
{
  ConnectWiFi();
  SyncNTPTime();
  DisconnectWiFi();

	if (m_inMenu) {
		m_inMenu = false;
		m_menu.Reset();
		DrawWatchFace(false);
	}
}

void WatchFace::MenuTimeZoneSelected(int tzOffset)
{
	m_tzOffset = tzOffset;

	if (m_inMenu) {
		m_inMenu = false;
		m_menu.Reset();
		DrawWatchFace(false);
	}
}

void WatchFace::MenuConfirmResetSteps()
{
  ResetSteps();

  if (m_inMenu) {
    m_inMenu = false;
    m_menu.Reset();
    DrawWatchFace(false);
  }
}
